@@ -218,7 +218,8 @@ panel_dbus_name_acquired (GDBusConnection *connection,
                           gpointer         user_data)
 {
   application = panel_application_get ();
-  panel_application_load (application, opt_disable_wm_check);
+  if (! panel_application_load (application, opt_disable_wm_check))
+    gtk_main_quit ();
 }
 
 
@@ -292,7 +293,7 @@ main (gint argc, gchar **argv)
       else
         g_print ("%s", PACKAGE_NAME);
       g_print (" %s (Xfce %s)\n\n", PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2004-2023");
+      g_print ("%s\n", "Copyright (c) 2004-2022");
       g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
       g_print ("\n");
