@@ -73,6 +73,7 @@ static void     pager_plugin_mode_changed                 (XfcePanelPlugin     *
                                                            XfcePanelPluginMode  mode);
 static void     pager_plugin_configure_workspace_settings (GtkWidget         *button);
 static void     pager_plugin_configure_plugin             (XfcePanelPlugin   *panel_plugin);
+static gpointer pager_plugin_get_master_plugin            (PagerPlugin       *plugin);
 static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin);
 static void     pager_plugin_get_preferred_width          (GtkWidget           *widget,
                                                            gint                *minimum_width,
@@ -108,9 +109,13 @@ struct _PagerPlugin
   guint          scrolling : 1;
   guint          wrap_workspaces : 1;
   guint          miniature_view : 1;
-  gint           rows;
+  guint          rows;
   gboolean       numbering;
   gfloat         ratio;
+
+  /* synchronize plugin with master plugin which manages workspace layout */
+  guint          sync_idle_id;
+  gboolean       sync_wait;
 };
 
 enum
@@ -123,6 +128,8 @@ enum
   PROP_NUMBERING
 };
 
+static GSList *plugin_list = NULL;
+
 
 
 /* define the plugin */
@@ -197,14 +204,25 @@ pager_plugin_class_init (PagerPluginClass *klass)
 static void
 pager_plugin_init (PagerPlugin *plugin)
 {
+  PagerPlugin *master_plugin;
+
   plugin->wnck_screen = NULL;
   plugin->scrolling = TRUE;
   plugin->wrap_workspaces = FALSE;
   plugin->miniature_view = TRUE;
-  plugin->rows = 1;
   plugin->numbering = FALSE;
   plugin->ratio = 1.0;
   plugin->pager = NULL;
+  plugin->sync_idle_id = 0;
+  plugin->sync_wait = TRUE;
+
+  master_plugin = pager_plugin_get_master_plugin (plugin);
+  if (master_plugin == NULL)
+    plugin->rows = 1;
+  else
+    plugin->rows = master_plugin->rows;
+
+  plugin_list = g_slist_append (plugin_list, plugin);
 }
 
 
@@ -229,8 +247,6 @@ pager_plugin_get_property (GObject    *object,
 
     case PROP_MINIATURE_VIEW:
       g_value_set_boolean (value, plugin->miniature_view);
-
-      pager_plugin_screen_layout_changed (plugin);
       break;
 
     case PROP_ROWS:
@@ -255,7 +271,8 @@ pager_plugin_set_property (GObject      *object,
                            const GValue *value,
                            GParamSpec   *pspec)
 {
-  PagerPlugin *plugin = XFCE_PAGER_PLUGIN (object);
+  PagerPlugin *plugin = XFCE_PAGER_PLUGIN (object), *master_plugin;
+  guint        rows;
 
   switch (prop_id)
     {
@@ -269,21 +286,45 @@ pager_plugin_set_property (GObject      *object,
 
     case PROP_MINIATURE_VIEW:
       plugin->miniature_view = g_value_get_boolean (value);
+      if (plugin->wnck_screen != NULL)
+        pager_plugin_screen_layout_changed (plugin);
       break;
 
     case PROP_ROWS:
-      plugin->rows = g_value_get_uint (value);
+      rows = g_value_get_uint (value);
+      if (rows == plugin->rows)
+        return;
 
-      if (plugin->pager != NULL)
+      plugin->rows = rows;
+      if (plugin->pager == NULL)
+        return;
+
+      master_plugin = pager_plugin_get_master_plugin (plugin);
+      if (plugin == master_plugin)
         {
+          /* set n_rows for master plugin and consequently workspace layout:
+           * this is delayed in both cases */
           if (plugin->miniature_view)
-            {
-              if (!wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows))
-                g_warning ("Failed to set the number of pager rows. You probably "
-                           "have more than 1 pager in your panel setup.");
-            }
+            wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
           else
             pager_buttons_set_n_rows (XFCE_PAGER_BUTTONS (plugin->pager), plugin->rows);
+
+          /* set n_rows for other plugins: this will queue a pager re-creation */
+          for (GSList *lp = plugin_list; lp != NULL; lp = lp->next)
+            if (lp->data != plugin
+                && XFCE_PAGER_PLUGIN (lp->data)->wnck_screen == plugin->wnck_screen)
+              g_object_set (lp->data, "rows", plugin->rows, NULL);
+        }
+      else
+        {
+          /* forward to master plugin first, else it is an internal call above */
+          if (master_plugin->rows != plugin->rows)
+            {
+              plugin->rows = 0;
+              g_object_set (master_plugin, "rows", rows, NULL);
+            }
+          else
+            pager_plugin_screen_layout_changed (plugin);
         }
       break;
 
@@ -434,6 +475,69 @@ pager_plugin_drag_end_event (GtkWidget      *widget,
 
 
 
+static gpointer
+pager_plugin_get_master_plugin (PagerPlugin *plugin)
+{
+  /* one master plugin per WnckScreen */
+  for (GSList *lp = plugin_list; lp != NULL; lp = lp->next)
+    if (XFCE_PAGER_PLUGIN (lp->data)->wnck_screen == plugin->wnck_screen)
+      return lp->data;
+
+  return NULL;
+}
+
+
+
+static void
+pager_plugin_set_ratio (PagerPlugin *plugin)
+{
+  WnckWorkspace *workspace;
+
+  g_signal_handlers_disconnect_by_func (plugin->wnck_screen, pager_plugin_set_ratio, plugin);
+
+  workspace = wnck_screen_get_active_workspace (plugin->wnck_screen);
+  if (workspace == NULL)
+    {
+      /* this is the right signal to get relevant information about the virtual
+       * nature of active workspace, instead of "active-workspace-changed" */
+      g_signal_connect_swapped (plugin->wnck_screen, "window-manager-changed",
+                                G_CALLBACK (pager_plugin_set_ratio), plugin);
+      return;
+    }
+
+  if (wnck_workspace_is_virtual (workspace))
+    plugin->ratio = (gfloat) wnck_workspace_get_width (workspace)
+                    / (gfloat) wnck_workspace_get_height (workspace);
+  else
+    plugin->ratio = (gfloat) wnck_screen_get_width (plugin->wnck_screen)
+                    / (gfloat) wnck_screen_get_height (plugin->wnck_screen);
+}
+
+
+
+static gboolean
+pager_plugin_screen_layout_changed_idle (gpointer data)
+{
+  PagerPlugin *plugin = data, *master_plugin;
+
+  /* changing workspace layout in buttons-view is delayed twice: in our code
+   * and in Libwnck code */
+  master_plugin = pager_plugin_get_master_plugin (plugin);
+  if (! master_plugin->miniature_view && plugin->sync_wait)
+    {
+      plugin->sync_wait = FALSE;
+      return TRUE;
+    }
+
+  pager_plugin_screen_layout_changed (plugin);
+  plugin->sync_wait = TRUE;
+  plugin->sync_idle_id = 0;
+
+  return FALSE;
+}
+
+
+
 static void
 pager_plugin_screen_layout_changed (PagerPlugin *plugin)
 {
@@ -443,6 +547,15 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin)
   panel_return_if_fail (XFCE_IS_PAGER_PLUGIN (plugin));
   panel_return_if_fail (WNCK_IS_SCREEN (plugin->wnck_screen));
 
+  /* changing workspace layout is delayed in Libwnck code, so we have to give time
+   * to the master plugin request to be processed */
+  if (plugin != pager_plugin_get_master_plugin (plugin) && plugin->sync_idle_id == 0)
+    {
+      plugin->sync_idle_id =
+        g_idle_add_full (G_PRIORITY_LOW, pager_plugin_screen_layout_changed_idle, plugin, NULL);
+      return;
+    }
+
   if (G_UNLIKELY (plugin->pager != NULL))
     {
       gtk_widget_destroy (GTK_WIDGET (plugin->pager));
@@ -456,12 +569,9 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin)
 
   if (plugin->miniature_view)
     {
+      pager_plugin_set_ratio (plugin);
+
       plugin->pager = wnck_pager_new ();
-      wnck_pager_set_display_mode (WNCK_PAGER (plugin->pager), WNCK_PAGER_DISPLAY_CONTENT);
-      wnck_pager_set_orientation (WNCK_PAGER (plugin->pager), orientation);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      plugin->ratio = (gfloat) gdk_screen_width () / (gfloat) gdk_screen_height ();
-G_GNUC_END_IGNORE_DEPRECATIONS
       g_signal_connect_after (G_OBJECT (plugin->pager), "drag-begin",
                               G_CALLBACK (pager_plugin_drag_begin_event), plugin);
       g_signal_connect_after (G_OBJECT (plugin->pager), "drag-end",
@@ -471,10 +581,11 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       g_signal_connect_swapped (G_OBJECT (plugin->pager), "scroll-event",
                                 G_CALLBACK (pager_plugin_scroll_event), plugin);
 
-      /* n_rows must be set after pager is anchored */
+      /* properties that change screen layout must be set after pager is anchored */
       gtk_container_add (GTK_CONTAINER (plugin), plugin->pager);
-      if (!wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows))
-        g_warning ("Setting the pager rows returned false. Maybe the setting is not applied.");
+      wnck_pager_set_display_mode (WNCK_PAGER (plugin->pager), WNCK_PAGER_DISPLAY_CONTENT);
+      wnck_pager_set_orientation (WNCK_PAGER (plugin->pager), orientation);
+      wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
     }
   else
     {
@@ -490,6 +601,8 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   /* Poke the style-updated signal to set the correct background color for the newly
      created widget. Otherwise it may sometimes end up transparent. */
   pager_plugin_style_updated (plugin->pager, NULL);
+  g_signal_connect (G_OBJECT (plugin->pager), "style-updated",
+      G_CALLBACK (pager_plugin_style_updated), NULL);
 }
 
 
@@ -503,9 +616,7 @@ pager_plugin_screen_changed (GtkWidget *widget,
   WnckScreen  *wnck_screen;
 
   screen = gtk_widget_get_screen (widget);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  wnck_screen = wnck_screen_get (gdk_screen_get_number (screen));
-G_GNUC_END_IGNORE_DEPRECATIONS
+  wnck_screen = wnck_screen_get (panel_screen_get_number (screen));
 
   if (plugin->wnck_screen != wnck_screen)
     {
@@ -513,10 +624,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
       pager_plugin_screen_layout_changed (plugin);
 
-      g_signal_connect_swapped (G_OBJECT (screen), "monitors-changed",
-         G_CALLBACK (pager_plugin_screen_layout_changed), plugin);
-      g_signal_connect_swapped (G_OBJECT (screen), "size-changed",
-         G_CALLBACK (pager_plugin_screen_layout_changed), plugin);
+      g_signal_connect_object (G_OBJECT (screen), "monitors-changed",
+         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
+      g_signal_connect_object (G_OBJECT (screen), "size-changed",
+         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
     }
 }
 
@@ -539,29 +650,23 @@ pager_plugin_construct (XfcePanelPlugin *panel_plugin)
 
   xfce_panel_plugin_menu_show_configure (panel_plugin);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  mi = gtk_image_menu_item_new_with_mnemonic (_("Workspace _Settings..."));
-G_GNUC_END_IGNORE_DEPRECATIONS
+  mi = panel_image_menu_item_new_with_mnemonic (_("Workspace _Settings..."));
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (mi));
   g_signal_connect (G_OBJECT (mi), "activate",
       G_CALLBACK (pager_plugin_configure_workspace_settings), NULL);
   gtk_widget_show (mi);
 
   image = gtk_image_new_from_icon_name ("org.xfce.panel.pager", GTK_ICON_SIZE_MENU);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-G_GNUC_END_IGNORE_DEPRECATIONS
+  panel_image_menu_item_set_image (mi, image);
   gtk_widget_show (image);
 
   panel_properties_bind (NULL, G_OBJECT (plugin),
                          xfce_panel_plugin_get_property_base (panel_plugin),
                          properties, FALSE);
 
+  pager_plugin_screen_changed (GTK_WIDGET (plugin), NULL);
   g_signal_connect (G_OBJECT (plugin), "screen-changed",
       G_CALLBACK (pager_plugin_screen_changed), NULL);
-  pager_plugin_screen_changed (GTK_WIDGET (plugin), NULL);
-  g_signal_connect (G_OBJECT (plugin->pager), "style-updated",
-      G_CALLBACK (pager_plugin_style_updated), NULL);
 }
 
 
@@ -573,6 +678,10 @@ pager_plugin_free_data (XfcePanelPlugin *panel_plugin)
 
   g_signal_handlers_disconnect_by_func (G_OBJECT (plugin),
       pager_plugin_screen_changed, NULL);
+
+  plugin_list = g_slist_remove (plugin_list, plugin);
+  if (plugin->sync_idle_id != 0)
+    g_source_remove (plugin->sync_idle_id);
 }
 
 
@@ -596,6 +705,9 @@ pager_plugin_mode_changed (XfcePanelPlugin     *panel_plugin,
   PagerPlugin       *plugin = XFCE_PAGER_PLUGIN (panel_plugin);
   GtkOrientation     orientation;
 
+  if (plugin->pager == NULL)
+    return;
+
   orientation =
     (mode != XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
     GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
