@@ -401,9 +401,7 @@ window_menu_plugin_screen_changed (GtkWidget *widget,
   /* get the wnck screen */
   screen = gtk_widget_get_screen (widget);
   panel_return_if_fail (GDK_IS_SCREEN (screen));
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  wnck_screen = wnck_screen_get (gdk_screen_get_number (screen));
-G_GNUC_END_IGNORE_DEPRECATIONS
+  wnck_screen = wnck_screen_get (panel_screen_get_number (screen));
   panel_return_if_fail (WNCK_IS_SCREEN (wnck_screen));
 
   /* leave when we same wnck screen was picked */
@@ -428,7 +426,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       G_CALLBACK (window_menu_plugin_active_window_changed), plugin);
 
   if (plugin->urgentcy_notification)
-     window_menu_plugin_windows_connect (plugin, FALSE);
+    window_menu_plugin_windows_connect (plugin, TRUE);
 }
 
 
@@ -623,13 +621,44 @@ window_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
 
 
 
+static void
+window_menu_plugin_set_icon (WindowMenuPlugin *plugin,
+                             WnckWindow       *window)
+{
+  GdkPixbuf *pixbuf;
+  gint       icon_size;
+
+  panel_return_if_fail (XFCE_IS_WINDOW_MENU_PLUGIN (plugin));
+  panel_return_if_fail (WNCK_IS_WINDOW (window));
+
+  if (! wnck_window_is_active (window))
+    return;
+
+  gtk_widget_set_tooltip_text (plugin->icon, wnck_window_get_name (window));
+
+  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
+  if (icon_size <= 31)
+    pixbuf = wnck_window_get_mini_icon (window);
+  else
+    pixbuf = wnck_window_get_icon (window);
+
+  if (G_LIKELY (pixbuf != NULL))
+    gtk_image_set_from_pixbuf (GTK_IMAGE (plugin->icon), pixbuf);
+  else
+    {
+      gtk_image_set_from_icon_name (GTK_IMAGE (plugin->icon), "image-missing", icon_size);
+      gtk_image_set_pixel_size (GTK_IMAGE (plugin->icon), icon_size);
+    }
+}
+
+
+
 static void
 window_menu_plugin_active_window_changed (WnckScreen       *screen,
                                           WnckWindow       *previous_window,
                                           WindowMenuPlugin *plugin)
 {
   WnckWindow     *window;
-  GdkPixbuf      *pixbuf;
   gint            icon_size;
   GtkWidget      *icon = GTK_WIDGET (plugin->icon);
   WnckWindowType  type;
@@ -639,7 +668,6 @@ window_menu_plugin_active_window_changed (WnckScreen       *screen,
   panel_return_if_fail (WNCK_IS_SCREEN (screen));
   panel_return_if_fail (plugin->screen == screen);
 
-  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
   /* only do this when the icon is visible */
   if (plugin->button_style == BUTTON_STYLE_ICON)
     {
@@ -651,27 +679,14 @@ window_menu_plugin_active_window_changed (WnckScreen       *screen,
           if (type == WNCK_WINDOW_DESKTOP || type == WNCK_WINDOW_DOCK)
             goto show_desktop_icon;
 
-          /* get the window icon and set the tooltip */
-          gtk_widget_set_tooltip_text (GTK_WIDGET (icon),
-                                       wnck_window_get_name (window));
-
-          if (icon_size <= 31)
-            pixbuf = wnck_window_get_mini_icon (window);
-          else
-            pixbuf = wnck_window_get_icon (window);
-
-          if (G_LIKELY (pixbuf != NULL))
-            gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
-          else {
-            gtk_image_set_from_icon_name (GTK_IMAGE (icon), "image-missing", icon_size);
-            gtk_image_set_pixel_size (GTK_IMAGE (icon), icon_size);
-          }
+          window_menu_plugin_set_icon (plugin, window);
         }
       else
         {
           show_desktop_icon:
 
           /* desktop is shown right now */
+          icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
           gtk_image_set_from_icon_name (GTK_IMAGE (icon), "user-desktop", icon_size);
           gtk_image_set_pixel_size (GTK_IMAGE (icon), icon_size);
           gtk_widget_set_tooltip_text (GTK_WIDGET (icon), _("Desktop"));
@@ -722,9 +737,11 @@ window_menu_plugin_window_opened (WnckScreen       *screen,
   panel_return_if_fail (plugin->screen == screen);
   panel_return_if_fail (plugin->urgentcy_notification);
 
-  /* monitor the window's state */
+  /* monitor some window properties */
   g_signal_connect (G_OBJECT (window), "state-changed",
       G_CALLBACK (window_menu_plugin_window_state_changed), plugin);
+  g_signal_connect_swapped (G_OBJECT (window), "icon-changed",
+      G_CALLBACK (window_menu_plugin_set_icon), plugin);
 
   /* check if the window needs attention */
   if (wnck_window_needs_attention (window))
@@ -767,13 +784,15 @@ window_menu_plugin_windows_disconnect (WindowMenuPlugin *plugin)
   g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->screen),
      window_menu_plugin_window_opened, plugin);
 
-  /* disconnect the state changed signal from all windows */
+  /* disconnect from all window signals */
   windows = wnck_screen_get_windows (plugin->screen);
   for (li = windows; li != NULL; li = li->next)
     {
       panel_return_if_fail (WNCK_IS_WINDOW (li->data));
       g_signal_handlers_disconnect_by_func (G_OBJECT (li->data),
           window_menu_plugin_window_state_changed, plugin);
+      g_signal_handlers_disconnect_by_func (G_OBJECT (li->data),
+          window_menu_plugin_set_icon, plugin);
     }
 
   /* stop blinking */
@@ -1008,9 +1027,7 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
     name = decorated = g_strdup_printf ("[%s]", name);
 
   /* create the menu item */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  mi = gtk_image_menu_item_new_with_label (name);
-G_GNUC_END_IGNORE_DEPRECATIONS
+  mi = panel_image_menu_item_new_with_label (name);
   gtk_widget_set_tooltip_text (mi, tooltip);
   g_object_set_qdata (G_OBJECT (mi), window_quark, window);
   g_signal_connect (G_OBJECT (mi), "button-release-event",
@@ -1068,9 +1085,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
           /* set the menu item label */
           image = gtk_image_new_from_pixbuf (pixbuf);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-G_GNUC_END_IGNORE_DEPRECATIONS
+          panel_image_menu_item_set_image (mi, image);
           gtk_widget_show (image);
 
           if (lucent != NULL)
@@ -1333,18 +1348,14 @@ window_menu_plugin_menu_new (WindowMenuPlugin *plugin)
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_show (mi);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      mi = gtk_image_menu_item_new_with_label (_("Add Workspace"));
-G_GNUC_END_IGNORE_DEPRECATIONS
+      mi = panel_image_menu_item_new_with_label (_("Add Workspace"));
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
           G_CALLBACK (window_menu_plugin_workspace_add), plugin);
       gtk_widget_show (mi);
 
       image = gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_MENU);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-G_GNUC_END_IGNORE_DEPRECATIONS
+      panel_image_menu_item_set_image (mi, image);
       gtk_widget_show (mi);
 
       if (G_LIKELY (workspace != NULL))
@@ -1361,9 +1372,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       else
         label = g_strdup_printf (_("Remove Workspace %d"), n_workspaces);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      mi = gtk_image_menu_item_new_with_label (label);
-G_GNUC_END_IGNORE_DEPRECATIONS
+      mi = panel_image_menu_item_new_with_label (label);
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_set_sensitive (mi, !!(n_workspaces > 1));
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1374,9 +1383,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       g_free (utf8);
 
       image = gtk_image_new_from_icon_name ("list-remove", GTK_ICON_SIZE_MENU);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-G_GNUC_END_IGNORE_DEPRECATIONS
+      panel_image_menu_item_set_image (mi, image);
       gtk_widget_show (mi);
     }
 
@@ -1388,6 +1395,15 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
 
 
+static gboolean
+window_menu_plugin_menu_reposition (gpointer data)
+{
+  gtk_menu_reposition (data);
+  return FALSE;
+}
+
+
+
 static void
 window_menu_plugin_menu (GtkWidget        *button,
                          WindowMenuPlugin *plugin)
@@ -1402,6 +1418,8 @@ window_menu_plugin_menu (GtkWidget        *button,
       && !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
     return;
 
+  menu = window_menu_plugin_menu_new (plugin);
+
   /* Panel plugin remote events don't send actual GdkEvents, so construct a minimal one so that
    * gtk_menu_popup_at_pointer/rect can extract a location correctly from a GdkWindow */
   if (gtk_get_current_event () == NULL)
@@ -1409,10 +1427,13 @@ window_menu_plugin_menu (GtkWidget        *button,
       event = g_slice_new0 (GdkEventButton);
       event->type = GDK_BUTTON_PRESS;
       event->window = gdk_get_default_root_window ();
+
+      /* reposition menu if popped up at widget and panel was autohidden when it was shown */
+      if (button != NULL)
+        g_idle_add (window_menu_plugin_menu_reposition, menu);
     }
 
   /* popup the menu */
-  menu = window_menu_plugin_menu_new (plugin);
   g_signal_connect (G_OBJECT (menu), "deactivate",
       G_CALLBACK (window_menu_plugin_menu_deactivate), plugin);
 
